/*----------------------------------------------------------------------------
 *  Copyright (C) 20078 Maxim Integrated Products Incorporated, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Maxim Integrated Products
 *  shall not be used except as stated in the Maxim Integrated Products
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 */
    
/** 
 * \file Serial8113.c
 * \brief Provides functions to initialize and putchar to serial port
 *
 * This file provides a function to initialize the serial port of the MAXQ2000
 * used on the DS8113 Evaluation Kit board. It also provides the low-level
 * function __putchar used by printf to output a single character on the
 * board's serial port.
 *
 */

#include <maxq2000.h>

//---------------------------------------------------------------------------
// Function: initSerial
//  Summary: Initializes serial port 0 in 10-bit asynchronous mode, 115200 baud.
//   Inputs: None
//   Output: None
//---------------------------------------------------------------------------

void
initSerial(void)
{
   SCON0 = (1<<6) | (1<<4);       // Set to mode 1 (10-bit asynchronous) and enable
   SMD0  = 2;                     // Baud rate = 16 x baud clock
   //PR0   = 0x45e8;                // P = 2^21 * 115200 / 13.5 MHz
   PR0   = 0x3afc;                // P = 2^21 * 115200 / 16.0 MHz
   SCON0 &= ~1;                   // Clear Receive character flag 
   SCON0 &= ~2;                   // Clear Transmit character flag
   
}

//---------------------------------------------------------------------------
// Function: __putchar
//  Summary: Outputs a single character over serial port 0.  This function is
//           used automatically by printf().
//   Inputs: int x - Character to output.
//   Output: None
//---------------------------------------------------------------------------

int
__putchar(int x)
{
   SBUF0 = x;                     // Output character over serial port
   while (!(SCON0 & 2));          // Wait for transmit complete flag to be set
   SCON0 &= ~2;                   // Clear transmit complete flag
}

