/*---------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 */

 /* 
 * \file Trans.c
 * \brief Function library for the ACOS3 APDU-based transaction demo
 *
 * This file contains functions required to initiate transaction operations
 * on an ACOS3 Smart Card using its operating system command APDUs
 *
 */

#include <stdio.h>
#include <string.h>
#include "maxq2000.h"
#include "lib8113.h"
#include "LCD_8113_Funct.h"
#include "Trans.h"

extern uint8_t LCD_Str[42];	// LCXD_Str (string to be displayed on LCD) defined in main

//
//
/* ************************************************************************** */
//  Function to initiate an ATR and receive the results
//

uint16_t transATR(void)
{
	int i ;

	// Get ATR from power-up
	rcvlen = dssc_getATRbuffer(rcvbuff,512);
	printf("ATR Status output: 0x%02X, 0x%02X\n", rcvbuff[rcvlen-2], rcvbuff[rcvlen-1]);
	printf("ATR:");
	for (i = 0;i < rcvlen;i++) printf(" 0x%02X",rcvbuff[i]);
	printf("\n");
	dssc_setNAD(0x00);

	// Check for error in ATR response	
	if ((rcvbuff[rcvlen-2] != 0x90) || (rcvbuff[rcvlen-1] != 0x00)){
		i = -1;
		transError(); }
	else i = 0 ;

	return(i) ;

}
/* ************************************************************************** */
//  Output an error message on the LCD.
//
void transError(void)
{
	int i ;

	strcpy(LCD_Str, "An ERROR occurred.");	// Establish first line of display
	LCD_WRCmd(DispClear);					// Clear the display
	i = LCD_Curpos (1,1);
	if (i == 0) LCD_WRStr(LCD_Str);			// Output string
		else printf("\n!! LCD cursor address out of range !!\n");
	delay (2);

	strcpy(LCD_Str, "Remove card and");		// Establish the first line of the display
	LCD_WRCmd(DispClear);					// Clear display
	i = LCD_Curpos (1,1);
	if (i == 0) LCD_WRStr(LCD_Str);			// Output string
		else printf("\n!! LCD cursor address out of range !!\n");
        strcpy(LCD_Str, "power down to reset.");	// Establish the second line of the display
	i = LCD_Curpos (2,1);
	if (i == 0) LCD_WRStr(LCD_Str);			// Output string
		else printf("\n!! LCD cursor address out of range !!\n");
	
	while(1);

}

/* ************************************************************************** */
//  Initialize the smart card to contain some pre-determined information.
//
uint8_t initCard(void)
{
	int i ;

	// Initialize the value of Count to 1.
	Count = 0x01 ;
	
	// Modify APDU for the initialization data (0x01)
	writePersRec2APDU[5] = Count;		// Set first byte of record (Count) to 1.
	for (i=0; i < 3; i++)			// Zero remaining 3 bytes of this record.
		writePersRec2APDU[i+6] = 0x0;

	// Initialize Account Balance to 100 (0x64)
	Bal = 0x64;
	loByteBal = Bal & 0xFF ;		// Break Bal into low and high bytes for storage
	hiByteBal = ((Bal >> 8) & 0xFF) ;
	// Modify APDU to contain the correct data
	writePersRec3APDU[5] = hiByteBal ;	// Update the "Bal" high byte of APDU
	writePersRec3APDU[6] = loByteBal ;	// Update the "Bal" low byte of APDU
	for (i=0; i < 2; i++)			// Zero last 2 bytes of APDU.
		writePersRec3APDU[i+7] = 0x0;	

	// Write modified APDUs to card
	i = writeRecords();

	// Display "Card Initialized" message on LCD
	strcpy(LCD_Str, "Card initialized.");
	LCD_WRCmd(DispClear);			// Clear the display
	i = LCD_Curpos (1,1);
	if (i == 0) LCD_WRStr(LCD_Str);		// Display message
		else printf("\n!! LCD cursor address out of range !!\n");
	delay(3) ;

	dispTrans() ;
	
	return(0) ;
}

/* ************************************************************************** */
//  Debit the card and update transaction count and balance
//
uint16_t transDebit(void)
{
	int i ;

	// Call routine to read card records and update Bal and Count variables
	i = readRecords();
   	
	Count++ ;
	printf("The new count = 0x%02X\n", Count);
	writePersRec2APDU[5] = Count;		// Update the "Count" byte of APDU
   	
	Bal = Bal - 10 ;			// Debit balance by $10
	printf("The new Balance = %d\n\n", Bal);
	loByteBal = Bal & 0xFF ; 		// Break Bal into low and high bytes for storage
	hiByteBal = ((Bal >> 8) & 0xFF) ;
	writePersRec3APDU[5] = hiByteBal;	// Update the "Bal" high byte of APDU
	writePersRec3APDU[6] = loByteBal;	// Update the "Bal" low byte of APDU

	// Write new values for Bal and Count to card
	i = writeRecords() ;

	// Display "Card debited" message on LCD
	strcpy(LCD_Str, "Card debited $10.00");
	LCD_WRCmd(DispClear);			// Clear Display
	i = LCD_Curpos (1,1);
	if (i == 0) LCD_WRStr(LCD_Str);		// Write message
		else printf("\n!! LCD cursor address out of range !!\n");
	delay(2) ;

	dispTrans() ;				// Display the Count and Bal

	return(0);

}
/* ************************************************************************** */
//  Credit the card and update transaction count and balance
//
uint16_t transCredit(void)
{
	int i ;

	// Call routine to read card records then update Bal and Count
	i = readRecords();

	Count++ ;				// Increment the transaction count
	printf("The new count = 0x%02X\n", Count);
	writePersRec2APDU[5] = Count;		// Update the "Count" byte of APDU

	Bal = Bal + 10 ;			// Credit the balance by $10
	printf("The new Balance = %d.\n\n", Bal);
	loByteBal = Bal & 0xFF ;		// Break Bal into low and high bytes for storage 
	hiByteBal = ((Bal >> 8) & 0xFF) ;
	writePersRec3APDU[5] = hiByteBal;	// Update the "Bal" low byte of APDU
	writePersRec3APDU[6] = loByteBal;	// Update the "Bal" high byte of APDU

	// Write new values for Bal and Count to card
	i = writeRecords();

	strcpy(LCD_Str, "Card credited $10.00");
	LCD_WRCmd(DispClear);			// Clear the display
	i = LCD_Curpos (1,1);
	if (i == 0) LCD_WRStr(LCD_Str);		// Write message
		else printf("\n!! LCD cursor address out of range !!\n");
	delay(2) ;

	dispTrans() ;				// Display the Count and Bal

	return(0);
}

/* ************************************************************************** */
//  Display the transaction count and balance on the LCD
//
void dispTrans(void)
{
	int i;

	LCD_WRCmd(DispClear);			// Clear display
	strcpy(LCD_Str, "Op Count = ");
	i = Count ;				// Convert count to an integer
	temp = sprintf(asciicnt, "%d", i);	// Convert integer to ascii
	strcat(LCD_Str, asciicnt);		// add to LCD string
	i = LCD_Curpos (1,1);
	if (i == 0) LCD_WRStr(LCD_Str);		// Write message
		else printf("\n!! LCD cursor address out of range !!\n");
	
	strcpy(LCD_Str, "Balance = ");
	temp = sprintf(asciibal, "$%d", Bal);	// Convert integer Bal to ascii
	strcat(LCD_Str, asciibal);		// add to LCD string
	strcat(LCD_Str, ".00");			// finish string
 	temp = LCD_Curpos (2,1);				
	if (temp == 0) LCD_WRStr(LCD_Str);	// Write message
		else printf("\n!! LCD cursor address out of range !!\n");
   	delay(3) ;

	return ;
}

/* ************************************************************************** */
//  Display a transaction-end message on the LCD
//
uint16_t transEnd(void)
{
	int i;

	LCD_WRCmd(DispClear);			// Clear the display
	strcpy(LCD_Str, "Thank you for");	// Create first line of message
	i = LCD_Curpos (1,1);			// and output
	if (i == 0) LCD_WRStr(LCD_Str);
		else printf("\n!! LCD cursor address out of range !!\n");
	
	strcpy(LCD_Str, "choosing the DS8113!"); // Create second line of display
	i = LCD_Curpos (2,1);			 // and output
	if (i == 0) LCD_WRStr(LCD_Str);
		else printf("\n!! LCD cursor address out of range !!\n");

	delay(2) ;

	LCD_WRCmd(DispClear);			// Clear display
	strcpy(LCD_Str, "Please remove card.");	// Create message string
	i = LCD_Curpos (1,1);			// and output
	if (i == 0) LCD_WRStr(LCD_Str);
		else printf("\n!! LCD cursor address out of range !!\n");
	
	return(0) ;
}

/* ************************************************************************** */
// Select the Personalization file 
//
//
uint16_t selectFile(void)
{
	int i ;

	// Select the Personaization File
	rcvlen = dssc_sendAPDU(selectPersfileAPDU, sizeof(selectPersfileAPDU),rcvbuff);
	printf("selectPersfileAPDU response length = %d ( ",rcvlen);
  	for (i = 0;i < rcvlen;i++)
		printf("0x%02X ",rcvbuff[i]);
	printf(")\n");
	if ((rcvbuff[rcvlen-2] != 0x90) || (rcvbuff[rcvlen-1] != 0x00))
		transError();

	return(0);

}

/* ************************************************************************** */
//  Read records 2 (transaction count) and 3 (balance) of the Personalization file.
//
uint16_t readRecords(void)
{
	int i ;

	// Read and print record 2 (of 3) of the Personaization File
	rcvlen = dssc_sendAPDU(readPersfilerec2APDU, sizeof(readPersfilerec2APDU),rcvbuff);
	printf("readPersfilerec2APDU response length = %d ( ",rcvlen);
  	for (i = 0;i < rcvlen;i++)
		printf("0x%02X ",rcvbuff[i]);
	printf(")\n");
	if ((rcvbuff[rcvlen-2] != 0x90) || (rcvbuff[rcvlen-1] != 0x00))
		transError();
	Count = rcvbuff[0] ;			// Get transaction count from file record
	printf("The Count read is 0x%02X\n", Count);
	writePersRec2APDU[5] = Count;           // Update the "Count" byte of APDU
	for (i=0; i < 3; i++)			// Copy old values from bytes 2, 3, & 4 into APDU
		writePersRec2APDU[i+6] = rcvbuff[i+1];

	// Read and print third (last) record of Personaization File
	rcvlen = dssc_sendAPDU(readPersfilerec3APDU, sizeof(readPersfilerec3APDU),rcvbuff);
	printf("readPersfilerec3APDU response length = %d ( ",rcvlen);
  	for (i = 0;i < rcvlen;i++)
		printf("0x%02X ",rcvbuff[i]);
	printf(")\n");
	if ((rcvbuff[rcvlen-2] != 0x90) || (rcvbuff[rcvlen-1] != 0x00))
		transError();
	
	hiByteBal = rcvbuff[0] ;		// Get high and low bytes of Bal from record
	loByteBal = rcvbuff[1] ;
	Bal = ((hiByteBal << 8)+loByteBal);	// Create 16-bit integer values from bytes.
	printf("The Balance read is %d\n\n", Bal);
	writePersRec3APDU[5] = hiByteBal;	// Update the "Bal" high byte of APDU
	writePersRec3APDU[6] = loByteBal;	// Update the "Bal" low byte of APDU
	for (i=0; i < 2; i++)			// Copy old values from bytes 3 & 4 into APDU
		writePersRec3APDU[i+7] = rcvbuff[i+2];

	return(0);

}

/* ************************************************************************** */
//  Write records 2 (transaction count) and 3 (balance) of the Personalization file.
//

uint16_t writeRecords(void)
{
	int i;

	// Write IC Code to allow write of records.
	rcvlen = dssc_sendAPDU(submitICcodeAPDU, sizeof(submitICcodeAPDU),rcvbuff);
	printf("submitICcodeAPDU response length = %d ( ",rcvlen);
	for (i = 0;i < rcvlen;i++)
		printf("0x%02X ",rcvbuff[i]);
	printf(")\n");
	if ((rcvbuff[rcvlen-2] != 0x90) || (rcvbuff[rcvlen-1] != 0x00))
		transError();

	// Write the new value of Count to the Personalization file	(record 2)
	rcvlen = dssc_sendAPDU(writePersRec2APDU, sizeof(writePersRec2APDU),rcvbuff);
	printf("writePersRec2APDU response length = %d ( ",rcvlen);
	for (i = 0;i < rcvlen;i++)
		printf("0x%02X ",rcvbuff[i]);
	printf(")\n");
	if ((rcvbuff[rcvlen-2] != 0x90) || (rcvbuff[rcvlen-1] != 0x00))
		transError();

	// Write the new value of Bal to the Personalization file (record 3)
	rcvlen = dssc_sendAPDU(writePersRec3APDU, sizeof(writePersRec3APDU),rcvbuff);
	printf("writePersRec3APDU response length = %d ( ",rcvlen);
	for (i = 0;i < rcvlen;i++)
		printf("0x%02X ",rcvbuff[i]);
	printf(")\n");
	if ((rcvbuff[rcvlen-2] != 0x90) || (rcvbuff[rcvlen-1] != 0x00))
		transError();

	// Read back record 2 to verify write
	rcvlen = dssc_sendAPDU(readPersfilerec2APDU, sizeof(readPersfilerec2APDU),rcvbuff);
	printf("readPersfilerec2APDU response length = %d ( ",rcvlen);
	for (i = 0;i < rcvlen;i++)
		printf("0x%02X ",rcvbuff[i]);
	printf(")\n");
	if ((rcvbuff[rcvlen-2] != 0x90) || (rcvbuff[rcvlen-1] != 0x00))
		transError();

	// Read back record 3 to verify write
	rcvlen = dssc_sendAPDU(readPersfilerec3APDU, sizeof(readPersfilerec3APDU),rcvbuff);
	printf("readPersfilerec3APDU response length = %d ( ",rcvlen);
	for (i = 0;i < rcvlen;i++)
		printf("0x%02X ",rcvbuff[i]);
	printf(")\n");
	if ((rcvbuff[rcvlen-2] != 0x90) || (rcvbuff[rcvlen-1] != 0x00))
		transError();

 	return(0);

}

