/*---------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 * ---------------------------------------------------------------------------
 */


/** 
 * \file Trans.h
 * \brief Header file for the Trans.c file
 *
 * This file contains information required
 * blah blah
 * blah blah
 *
 */

/**
 * \brief   delay (i.e., loop) by secs seconds.
 * \param   SECS number of seconds to delay
 *
 * \return  nothing.
 */
void delay(int secs);

/**
 * \brief   transError An error occurred in the transaction. Display message and exit.
 * \param   none.
 *
 * \return  none.
 */
void transError(void);

/**
 * \brief   dispTrans Debit the card and update the transaction count.
 * \param   none.
 *
 * \return  none.
 */
void dispTrans(void) ;

/**
 * \brief   transDebit Debit the card and update the transaction count.
 * \param   none.
 *
 * \return  uint16_t status code.
 */
uint16_t transDebit(void);

/**
 * \brief   transDebit Debit the card and update the transaction count.
 * \param   none.
 *
 * \return  uint16_t.
 */
uint16_t transCredit(void);

/**
 * \brief   selectFile Select the Personalization file on the card
 * \param   none.
 *
 * \return  uint16_t status code.
 */
uint16_t selectFile(void) ;

/**
 * \brief   readRecords Read records 2 and 3 of the Personalization file
 * \param   none.
 *
 * \return  uint16_t status code.
 */
uint16_t readRecords(void) ;

/**
 * \brief   writeRecords Write records 2 and 3 of the Personalization file.
 * \param   none.
 *
 * \return  uint16_t status code.
 */
uint16_t writeRecords(void) ;

/**
 * \brief   transEnd Display the ending transaction message on the LCD
 * \param   none.
 *
 * \return  uint16_t status code.
 */
uint16_t transEnd(void) ;

/* define bit location registers */
//sbit	P32 = 0xB2;				// P32/INT = Port 3.2

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* ACOS3 APDU operation definitions                        */
/*                                                         */

// Start session (generate Random Number) APDU
uint8_t startsessionAPDU[] = { 0x80,0x84,0,0,8 };

// MCU-ID File APDUs
uint8_t selectMCUfileAPDU[] = { 0x80,0xA4,0,0,0x02,0xFF, 0x00 };
uint8_t readMCUrecord1APDU[] = { 0x80,0xB2,0,0,8 };
uint8_t readMCUrecord2APDU[] = { 0x80,0xB2,1,0,8 };

// Manufacturer File APDUs
uint8_t selectMfgfileAPDU[] = { 0x80,0xA4,0,0,0x02,0xFF,1 };
uint8_t readMfgrecord1APDU[] = { 0x80,0xB2,0,0,0x08 };
uint8_t readMfgrecord2APDU[] = { 0x80,0xB2,1,0,0x08 };

// Personalization File APDUs
uint8_t selectPersfileAPDU[] = { 0x80,0xA4,0,0,0x02,0xFF,2 };
uint8_t readPersfilerec1APDU[] = { 0x80,0xB2,0,0,0x04 };
uint8_t readPersfilerec2APDU[] = { 0x80,0xB2,1,0,0x04 };
uint8_t readPersfilerec3APDU[] = { 0x80,0xB2,2,0,0x04 };
uint8_t writePersRec1APDU[] = { 0x80,0xD2,0,0,0x04,0,0,0,0 };
uint8_t writePersRec2APDU[] = { 0x80,0xD2,1,0,0x04,0,0,0,0 };
uint8_t writePersRec3APDU[] = { 0x80,0xD2,2,0,0x04,0,0,0,0 };

// Submit IC Code security code 
uint8_t submitICcodeAPDU[] = { 0x80,0x20,7,0,8,0x41,0x43,0x4F,0x53,0x54,0x45,0x53,0x54 };

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* Common variable definitions                             */
/*                                                         */

// Common variable definitions
	uint8_t rcvbuff[128], Count, loByteBal, hiByteBal;
	int16_t rcvlen;
	int Bal, temp;
	char asciicnt[10], asciibal[10];

